package kr.co.mkblog.mkblogtriangleonecolor;

import android.opengl.GLES30;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

/**
 * Created by mkblog on 2018-05-30.
 */

public class triangle {
    private String mkShaderCode = "";
    private String mkFragCode = "";

    //MK: Triangle Vertex
    private float[] mkVertices = {  -0.5f, -0.5f, 0.0f, // left
                                    0.5f, -0.5f, 0.0f, // right
                                    0.0f,  0.5f, 0.0f // top
                                };

    private int mkVertexShader;
    private int mkFragShader;
    private int mkProgram;

    private int[] mkVAO;
    private int[] mkVBO;


    public triangle(){
        //MK: Shader Code
        mkShaderCode =      "#version 300 es \n"
                        +   "layout (location = 0) in vec3 aPos; "
                        +   " void main() {"
                        +   "gl_Position = vec4(aPos.x, aPos.y, aPos.z, 1.0); }";
        //MK: Fragement Code
        mkFragCode =      "#version 300 es \n"
                        +   "out highp vec4 FragColor;"
                        +   "void main() {"
                        +   "FragColor = vec4(1.0f, 0.5f, 0.2f, 1.0f);}";

        //MK: Shader Compile
        mkVertexShader = GLES30.glCreateShader(GLES30.GL_VERTEX_SHADER);
        GLES30.glShaderSource(mkVertexShader, mkShaderCode);
        GLES30.glCompileShader(mkVertexShader);

        //MK: Fragment Compile
        mkFragShader = GLES30.glCreateShader(GLES30.GL_FRAGMENT_SHADER);
        GLES30.glShaderSource(mkFragShader, mkFragCode);
        GLES30.glCompileShader(mkFragShader);

        //MK: Create GL Program and Attach Shader and Fragment Code
        mkProgram = GLES30.glCreateProgram();
        GLES30.glAttachShader(mkProgram, mkVertexShader);
        GLES30.glAttachShader(mkProgram, mkFragShader);
        GLES30.glLinkProgram(mkProgram);
        GLES30.glDeleteShader(mkVertexShader);
        GLES30.glDeleteShader(mkFragShader);

        //MK: Make VAO, VBO
        mkVAO = new int[1];
        GLES30.glGenVertexArrays(1, mkVAO, 0);

        mkVBO = new int[1];
        GLES30.glGenBuffers(1, mkVBO, 0);

        GLES30.glBindVertexArray(mkVAO[0]);

        GLES30.glBindBuffer(GLES30.GL_ARRAY_BUFFER, mkVBO[0]);

        //MK: Loading Vertices onto GPUs
        ByteBuffer mkByteBuffer = ByteBuffer.allocateDirect(mkVertices.length * 4);
        mkByteBuffer.order(ByteOrder.nativeOrder());
        FloatBuffer mkVertexBuffer = mkByteBuffer.asFloatBuffer();
        mkVertexBuffer.put(mkVertices);
        mkVertexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ARRAY_BUFFER, mkVertices.length*4, mkVertexBuffer, GLES30.GL_STATIC_DRAW );
        GLES30.glVertexAttribPointer(0, 3, GLES30.GL_FLOAT, false, 3 * 4, 0);
        GLES30.glEnableVertexAttribArray(0);

    }

    public void draw(){
        //MK: Draw Triangle
        GLES30.glUseProgram(mkProgram);
        GLES30.glBindVertexArray(mkVAO[0]);
        GLES30.glDrawArrays(GLES30.GL_TRIANGLES, 0, 3);
    }
}
